/******************************************************************/
/*                                                                */
/*   Include file for K5/VSSP and K5/VSSP32 samplers              */
/*            libipvlbi_board.h                                   */
/*                                                                */
/*      Version:    2007-08-23                                    */
/*                                                                */
/*      Copyright (c) 2002-2007 T.Kondo/NICT All Right Reserved   */
/*                                                                */
/******************************************************************/
/*                                   
  K5/VSSP (IP-VLBI) and K5/VSSP32 sampler board  functions
         by T.Kondo 2001.8.26      
                    2002.5.26      
                    2002.10.21  sampleget was modified to handle file > 2GB      
                    2003.6.18   LogWrite2, LogWrite3 added
                    2003.8.09   miner message change in sampleget
                    2003.8.21   toffset_get is added
                    2004.5.23   LogWrite4 was added
                                sampleget was newnewed (use of structual variable as 
                                input parameter, and add log out function)
                                former sampleget was renamed as sampleget_old
                    2005.2.12   samplget was modified to more robust against disk full
                    2005.2.17   debug in samplget (cbuf[81]->cbuf[120]) to avoid
                                the error occurred at the case of very long file name
                    2005.2.12   libipvlbi_board.h was devided into *.h and *.c
                    2005-09-28  support linux2.6
                    2005-12-20  support VSSP32
                    2006-11-21  add SMPLING_RETRY
                    2007-02-14  delete functions as follows which are no more use
                                  sampleget_big, sampleget_old, sampleget0, 
                                  sampleget2, sampleget3
                    2007-02-22  flag_vssp32 was added as a member of
                                vssp_param type valuable
                    2007-02-23  board_ref_check, board_ref_check2, board_time_disp were added
                    2007-04-11 #include <sys/tdssdh.h> was commented out (never use in K5 utilities)
                                use of this will result in 
                                 #warning using private kernel header; include <endian.h> instead
                    2007-04-17  consider in case of C++ 
                    2007-08-23  definition of _VSSP32_OLD_DRIVER was added to avoid an compiler error 
                                in case of the use of old VSSP32 driver (VSSP32 driver version before 2007-08-17)
                                "int bitshift" and "int fpga_version"  were added as a member of vssp_param type variable.
                                function "get_fpga_version" was added.  
*/
#ifndef _LIBIPVLBI_BOARD_
#define _LIBIPVLBI_BOARD_


#include "chead.h"  // common header (src/chead.h)
#include "libipvlbi.h"

#ifdef linux
  #include <stdint.h>  // necessary for handling uint32_t etc.
  #include <sys/ioctl.h>
  #ifdef _VSSP32
    #include <sys/tdsio.h>
    //#include <sys/tdssdh.h>   // 2007-04-11
  #else
    #include <linux/tdsio.h>
  #endif
#else 
  #include <sys/tdsio.h>  // IP-VLBI board header
#endif

/* device driver file set  (added on 2005-12-20) */
#ifdef _VSSP32
  #define DEV_FILE "/dev/utds0"     // device file for K5/VSSP32
  #define SAMPLING_RETRY 20         // # of sampler retry when error occured
#else
  #define DEV_FILE "/dev/tds0"      // device file for K5/VSSP
  #define SAMPLING_RETRY 10         // # of sampler retry when error occured
#endif


/* following definitions is for old driver of which version is before  2007-08-17 */  // added on 2007-08-23
#ifdef _VSSP32
  #ifndef TDSIO_GET_VERFPGA        // representative of TDSIO_SET_BITSHIFT, TDSIO_GET_BITSHIFT, TDSIO_GET_VERFPGA
    #ifndef _VSSP32_OLD_DRIVER
       #define _VSSP32_OLD_DRIVER
    #endif
  #endif
#endif



#ifndef _LIBIPVLBI_
typedef struct listoutdir    // (moved to libipvlbi.h)
{            
    int  dirnum;   // maximum number of directories
    int  ndir;     // next directory index#  2005-01-30
    char dir[10][120];
} listoutdir_t;

typedef struct logval    // structural variablle for log  (moved to libipvlbi.h)
{            
    char logdir[60];     // logfile directory
    char hostname[20];   // host name
    char logfile[80];    //log file name
    int logfiletype;     // logfile naming type
                         //  0: exocode_stationid.hostname.log
                         //  1: YYYY+hostname+".log"
                         //       where  YYYY is 4 digit year
                         //  -1: no log
} logval_t;

typedef struct vsspaux  // VSSP32 AUX field type  for K5/VSSP32  (moved to libipvlbi.h)
{
    int size;    // AUX filed size
    int fmt;     // AUX field format
    char bdat[256];   // AUX field data
    char sid[2];        // 1 char Station ID
    char sid2[3];       // 2 char Station ID
    char station8[20];       // station name (1st 8 char is valid)
    char hostname8[20];       // Host PC name (1st 8 char is valid)
} vsspaux_t;

typedef struct vssp_param   // sampling parameter type variable  (moved to libipvlbi.h)
{
    int sfreq;
    int adbit;
    int bitshift;  // bit shift parameter in case of 2 or 4 bit sampling  2007-08-23
    int numch;
    int filter;  // filter frequency(MHz)  0: no use of K5/VSSP32
    int freqkhz;  // frequency in kHz  (used for only return param)
    int flag_vssp32;  // 0: K5/VSSP   1: K5/VSSP32
    int fpga_version; // FPGA Version for VSSP32  ( Major Version *100 + Minor Version ) 2007-08-23
    vsspaux_t vsspaux;  // AUX filed data for K5/VSSP32 2005-12-28
                        // vsspaux_t is defined in libipvlbi.h
} vssp_param_t;

typedef struct spget   // sampleget type variable (moved to libipvlbi.h)
{
    int span;
    //int sfreq;
    //int adbit;
    //int numch;
    //int filter;  // filter frequency(MHz)  0: no use of K5/VSSP32
    vssp_param_t sp; 
    int f_size_mode;  // 0: no limitation on file size  
                      // 1: divided into 2GB size 
    char outfile[120];
    char fname[120]; 
    char outdir[120];
    listoutdir_t outdirtab;  //2005-01-30
    logval_t lg;
    //vsspaux_t vsspaux;  // AUX filed data for K5/VSSP32 2005-12-28
                        // vsspaux_t is defined in libipvlbi.h
} spget_t;
#endif


#ifdef __cplusplus
extern "C"{
#endif 

int ini_sampler(int *fd_in, vssp_param_t *sp, int *bitsec);
int start_sampler(int *fd_in);
int stop_sampler(int *fd_in);
int ini_and_start_sampler(int *fd_in, vssp_param_t *sp, int *bitsec);
int sampleget(spget_t *spg);
int sampleget4(spget_t *spg, int mode);
int ip_board_time(unsigned int *day, unsigned int *sec, int *hh, int *mm, int *ss);
int toffset_get(int *yearPC, float *tofset); 
int LogWrite2(char* logdir, char* hostname, int time_mode, 
              float secofset, int mode, char* cbuf); 
void LogWrite3(char* logdir, char* hostname, char* logfile, int logfiletype,
              int time_mode,  float secofset, int mode, char* cbuf);
void LogWrite4(logval_t *lg, int time_mode,  float secofset, int mode, 
               char* cbuf);

int check_vssp(int fd_in, unsigned int *stat, int verbose);
int set_vssp_sparam(int fd_in, vssp_param_t *sp, int verbose);
int get_vssp_sparam(int fd_in, vssp_param_t *sp, int verbose);
int vssp_wait_nsec(int fd_in, int nsec);

/****** added 2007-02-23 ****/
unsigned int board_ref_check(int fd_in);
int board_ref_check2(int fd_in);
int board_time_disp(int secmax);
int get_fpga_version(void);

#ifdef __cplusplus
}
#endif


#endif
